@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2007, 2008, 2010, 2011 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2024 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,7 +30,8 @@
                BAD - command unknown or arguments invalid
 */
 int
-imap4d_examine (struct imap4d_command *command, imap4d_tokbuf_t tok)
+imap4d_examine (struct imap4d_session *session,
+                struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
   if (imap4d_tokbuf_argc (tok) != 3)
     return io_completion_response (command, RESP_BAD, "Invalid arguments");
