@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2005-2007, 2009-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2024 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -62,7 +61,7 @@ extern void mu_scm_address_init (void);
 extern void mu_scm_logger_init (void);
 
 extern void mu_scm_port_init (void);
-extern SCM mu_port_make_from_stream (SCM msg, mu_stream_t stream, long mode);
+extern SCM mu_port_make_from_stream (mu_stream_t stream, long mode);
 
 extern void mu_scm_mime_init (void);
 extern void mu_scm_message_add_owner (SCM MESG, SCM owner);
@@ -84,6 +83,8 @@ extern int mu_guile_safe_exec (SCM (*handler) (void *data), void *data,
 			       SCM *result);
 extern int mu_guile_safe_proc_call (SCM proc, SCM arglist, SCM *presult);
 
+extern void mu_scm_filter_init (void);
+	
 #ifdef __cplusplus
 }
 #endif
