@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009-2020 Free Software Foundation, Inc.
+   Copyright (C) 2009-2024 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,8 +17,9 @@
 #ifndef _MAILUTILS_STREAM_H
 #define _MAILUTILS_STREAM_H
 
-#include <mailutils/types.h>
+#include <stdlib.h>
 #include <stdarg.h>
+#include <mailutils/types.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -79,7 +80,10 @@ enum mu_buffer_type
 #define MU_IOCTL_TLSSTREAM       13 /* TLS stream */
 #define MU_IOCTL_WORDWRAPSTREAM  14 /* Word-wrapper stream */
 #define MU_IOCTL_TCPSTREAM       15 /* TCP stream */
-  
+
+#define MU_IOCTL_TIMEOUT         16 /* Get or set the I/O timeout value
+				       (struct timeval) */
+
   /* Opcodes common for various families */
 #define MU_IOCTL_OP_GET 0
 #define MU_IOCTL_OP_SET 1  
@@ -187,7 +191,7 @@ enum mu_buffer_type
      New transcript level is set to *X.
      If setting separate levels for each channel, use MU_XSCRIPT_LEVEL_PACK
      macro to pack them into one integer value. Use
-     MU_IOCTL_XSCRIPTSTREAM_CHANNEL to confugure single channel.
+     MU_IOCTL_XSCRIPTSTREAM_CHANNEL to configure single channel.
      
      Upon successful return, previous levels are stored in X. Use the
      MU_XSCRIPT_LEVEL_UNPACK macro to retrieve level for a particular
@@ -337,13 +341,54 @@ int mu_stream_set_buffer (mu_stream_t stream, enum mu_buffer_type type,
 int mu_stream_get_buffer (mu_stream_t stream,
 			  struct mu_buffer_query *qry);
 int mu_stream_read (mu_stream_t stream, void *buf, size_t size, size_t *pread);
-int mu_stream_readdelim (mu_stream_t stream, char *buf, size_t size,
-			 int delim, size_t *pread);
-int mu_stream_readline (mu_stream_t stream, char *buf, size_t size, size_t *pread);
-int mu_stream_getdelim (mu_stream_t stream, char **pbuf, size_t *psize,
-			int delim, size_t *pread);
-int mu_stream_getline (mu_stream_t stream, char **pbuf, size_t *psize,
-		       size_t *pread);
+int mu_stream_timed_readdelim (mu_stream_t stream, char *buf, size_t size,
+			       int delim, struct timeval *to, size_t *pread);
+static inline int
+mu_stream_readdelim (mu_stream_t stream, char *buf, size_t size,
+		     int delim, size_t *pread)
+{
+  return mu_stream_timed_readdelim (stream, buf, size, delim, NULL, pread);
+}
+
+static inline int
+mu_stream_timed_readline (mu_stream_t stream, char *buf, size_t size,
+			  struct timeval *tv,
+			  size_t *pread)
+{
+  return mu_stream_timed_readdelim (stream, buf, size, '\n', tv, pread);
+}
+
+static inline int
+mu_stream_readline (mu_stream_t stream, char *buf, size_t size, size_t *pread)
+{
+  return mu_stream_timed_readline (stream, buf, size, NULL, pread);
+}
+
+int mu_stream_timed_getdelim (mu_stream_t stream, char **pbuf, size_t *psize,
+			      int delim, struct timeval *to, size_t *pread);
+
+static inline int
+mu_stream_getdelim (mu_stream_t stream, char **pbuf, size_t *psize,
+		    int delim, size_t *pread)
+{
+  return mu_stream_timed_getdelim (stream, pbuf, psize, delim, NULL, pread);
+}
+
+static inline int
+mu_stream_timed_getline (mu_stream_t stream, char **pbuf, size_t *psize,
+			 struct timeval *to,
+			 size_t *pread)
+{
+  return mu_stream_timed_getdelim (stream, pbuf, psize, '\n', to, pread);
+}
+
+static inline int
+mu_stream_getline (mu_stream_t stream, char **pbuf, size_t *psize,
+		   size_t *pread)
+{
+  return mu_stream_timed_getline (stream, pbuf, psize, NULL, pread);
+}
+
 int mu_stream_write (mu_stream_t stream, const void *buf, size_t size,
 		     size_t *pwrite);
 int mu_stream_writeline (mu_stream_t stream, const char *buf, size_t size);
@@ -386,7 +431,7 @@ int mu_file_stream_create (mu_stream_t *pstream, const char *filename, int flags
 struct mu_tempfile_hints;  
 int mu_temp_file_stream_create (mu_stream_t *pstream,
 				struct mu_tempfile_hints *hints, int flags);
-int mu_fd_stream_create (mu_stream_t *pstream, char *filename, int fd,
+int mu_fd_stream_create (mu_stream_t *pstream, char const *filename, int fd,
 			 int flags);
 
 #define MU_STDIN_FD  0
@@ -424,6 +469,8 @@ int mu_tcp_stream_create_with_source_host (mu_stream_t *stream,
 int mu_tcp_stream_create (mu_stream_t *stream, const char *host, unsigned port,
 			  int flags);
 
+int mu_mailbox_stream_create (mu_stream_t *pstream, const char *filename, int flags);
+
 /* Transcript log levels */
 #define MU_XSCRIPT_NORMAL  0  /* Normal transcript */
 #define MU_XSCRIPT_SECURE  1  /* Security-related data filtered out */
