@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2000, 2005, 2007, 2010-2012, 2014-2018 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2024 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -37,6 +36,7 @@ struct _mu_observer
 struct _mu_observable
 {
   void *owner;
+  size_t types;
   mu_list_t list;
 };
 
