@@ -1,6 +1,6 @@
 /* utmp.c -- Replacements for {set,get,end}utmp functions
 
-Copyright (C) 2002 Free Software Foundation, Inc.
+Copyright (C) 2002-2024 Free Software Foundation, Inc.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
@@ -13,18 +13,21 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
-#include <sys/types.h>
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+#include <sys/types.h> 
 #include <sys/time.h>
 #include <utmp.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <string.h>
 #include <stdio.h>
+#include <confpaths.h>
 
-static char *utmp_name = _PATH_UTMP;
+static char *utmp_name = PATH_UTMP;
 static int fd = -1;
 static struct utmp ut;
 
@@ -56,4 +59,3 @@ getutent ()
 
   return &ut;
 }
-
