@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2022 Free Software Foundation, Inc.
+   Copyright (C) 1999-2024 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -47,8 +47,6 @@ mu_fgetpwent_r (FILE *fp, char **bufp, size_t *bufs, struct passwd **result)
 {
   char *buffer = *bufp;
   size_t buflen = *bufs;
-  char *nb;
-  size_t ns;
   struct passwd *pwbuf;
   size_t pos = SPWDSIZ;
   int c;
@@ -63,18 +61,16 @@ mu_fgetpwent_r (FILE *fp, char **bufp, size_t *bufs, struct passwd **result)
       if (pos >= buflen)
 	{
 	  if (buflen == 0)
-	    ns = SPWDSIZ + 128;
+	    buflen = SPWDSIZ + 128;
 	  else
 	    {
-	      ns = ns * 2;
-	      if (ns < buflen)
+	      if ((size_t) -1 / 3 * 2 <= buflen)
 		return ENOMEM;
+	      buflen += (buflen + 1) / 2;
 	    }
-	  nb = realloc (buffer, ns);
-	  if (!nb)
+	  buffer = realloc (buffer, buflen);
+	  if (!buffer)
 	    return ENOMEM;
-	  buffer = nb;
-	  buflen = ns;
 	}
       if (c == '\n')
 	{
@@ -105,11 +101,10 @@ mu_fgetpwent_r (FILE *fp, char **bufp, size_t *bufs, struct passwd **result)
 	return ENOENT;
       if (pos == buflen)
 	{
-	  nb = realloc (buffer, buflen + 1);
-	  if (!nb)
+	  buflen++;
+	  buffer = realloc (buffer, buflen);
+	  if (!buffer)
 	    return ENOMEM;
-	  buffer = nb;
-	  buflen = ns;
 	}
       buffer[pos++] = 0;
     }
