@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2004, 2005, 2006, 2007, 2008, 2009,
-   2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 1999-2024 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -23,6 +22,7 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
+#include <unistd.h>
 
 #include <mailutils/debug.h>
 #include <mailutils/errno.h>
@@ -41,13 +41,9 @@
 #include <mailutils/util.h>
 
 #include <mailutils/sys/mailbox.h>
+#include <mailutils/sys/folder.h>
 #include <mailutils/sys/url.h>
 
-/* Mailbox-specific flags */
-#define _MU_MAILBOX_OPEN    0x10000000
-#define _MU_MAILBOX_REMOVED 0x20000000
-#define _MU_MAILBOX_MASK    0xF0000000
-
 static int
 mailbox_folder_create (mu_mailbox_t mbox, const char *name,
 		       mu_record_t record)
@@ -76,7 +72,8 @@ mailbox_folder_create (mu_mailbox_t mbox, const char *name,
 int
 _mailbox_create_from_record (mu_mailbox_t *pmbox,
 			     mu_record_t record,
-			     mu_url_t url, 
+			     mu_url_t url,
+			     mu_folder_t folder,
 			     const char *name)
 {
   int (*m_init) (mu_mailbox_t) = NULL;
@@ -92,6 +89,7 @@ _mailbox_create_from_record (mu_mailbox_t *pmbox,
       mbox = calloc (1, sizeof (*mbox));
       if (mbox == NULL)
 	return ENOMEM;
+      mbox->notify_fd = -1;
       
       /* Initialize the internal lock now, so the concrete mailbox
 	 could use it. */
@@ -127,10 +125,16 @@ _mailbox_create_from_record (mu_mailbox_t *pmbox,
 	}
       
       mbox->url = url;
-      
-      /* Create the folder before initializing the concrete mailbox.
-	 The mailbox needs it's back pointer. */
-      status = mailbox_folder_create (mbox, name, record);
+
+      if (folder)
+	{
+	  folder->ref++; /* FIXME: No ref/unref function for folders */
+	  mbox->folder = folder;
+	}
+      else
+	/* Create the folder before initializing the concrete mailbox.
+	   The mailbox needs it's back pointer. */
+	status = mailbox_folder_create (mbox, name, record);
       
       if (status == 0)
 	status = m_init (mbox);   /* Create the concrete mailbox type.  */
@@ -152,11 +156,12 @@ static int
 _create_mailbox0 (mu_mailbox_t *pmbox, mu_url_t url, const char *name)
 {
   mu_record_t record = NULL;
-
-  if (mu_registrar_lookup_url (url, MU_FOLDER_ATTRIBUTE_FILE, &record, NULL)
-      == 0)
-    return _mailbox_create_from_record (pmbox, record, url, name);
-  return ENOSYS;
+  int rc;
+  
+  rc = mu_registrar_lookup_url (url, MU_FOLDER_ATTRIBUTE_FILE, &record, NULL);
+  if (rc == 0)
+    rc = _mailbox_create_from_record (pmbox, record, url, NULL, name);
+  return rc;
 }
 
 static int
@@ -203,7 +208,71 @@ mu_mailbox_create_from_record (mu_mailbox_t *pmbox, mu_record_t record,
   rc = mu_url_create (&url, name);
   if (rc)
     return rc;
-  rc = _mailbox_create_from_record (pmbox, record, url, name);
+  rc = _mailbox_create_from_record (pmbox, record, url, NULL, name);
+  if (rc)
+    mu_url_destroy (&url);
+  return rc;
+}
+
+int
+mu_mailbox_create_at (mu_mailbox_t *pmbox, mu_folder_t folder,
+		      const char *name)
+{
+  int rc;
+  mu_url_t url;
+  const char *oldpath;
+  
+  rc = mu_url_dup (folder->url, &url);
+  if (rc)
+    return rc;
+  do
+    {
+      char *path;
+      size_t oldlen, len;
+      mu_record_t record;
+      
+      rc = mu_url_sget_path (url, &oldpath);
+      if (rc)
+	break;
+
+      oldlen = strlen (oldpath);
+      if (oldlen == 0)
+	{
+	  path = strdup (name);
+	  if (!path)
+	    {
+	      rc = ENOMEM;
+	      break;
+	    }
+	}
+      else
+	{
+	  if (oldpath[oldlen-1] == '/')
+	    oldlen--;
+	  len = oldlen + 1 + strlen (name) + 1;
+	  path = malloc (len);
+	  if (!path)
+	    {
+	      rc = ENOMEM;
+	      break;
+	    }
+	  memcpy (path, oldpath, oldlen);
+	  path[oldlen++] = '/';
+	  strcpy (path + oldlen, name);
+	}
+      rc = mu_url_set_path (url, path);
+      free (path);
+      if (rc)
+	break;
+
+      rc = mu_registrar_lookup_url (url, MU_FOLDER_ATTRIBUTE_FILE,
+				    &record, NULL);
+      if (rc)
+	break;
+      rc = _mailbox_create_from_record (pmbox, record, url, folder, name);
+    }
+  while (0);
+
   if (rc)
     mu_url_destroy (&url);
   return rc;
@@ -240,18 +309,13 @@ mu_mailbox_destroy (mu_mailbox_t *pmbox)
 	  mu_stream_destroy (&mbox->stream);
 	}
 
-      if (mbox->url)
-        mu_url_destroy (&mbox->url);
-
-      if (mbox->locker)
-	mu_locker_destroy (&mbox->locker);
-
-      if (mbox->folder)
-	mu_folder_destroy (&mbox->folder);
-
-      if (mbox->property)
-	mu_property_destroy (&mbox->property);
-
+      mu_url_destroy (&mbox->url);
+      mu_locker_destroy (&mbox->locker);
+      mu_folder_destroy (&mbox->folder);
+      mu_property_destroy (&mbox->property);
+      free (mbox->notify_user);
+      free (mbox->notify_sa);
+      
       free (mbox);
       *pmbox = NULL;
       mu_monitor_unlock (monitor);
@@ -300,7 +364,11 @@ mu_mailbox_close (mu_mailbox_t mbox)
 
   rc = mbox->_close (mbox);
   if (rc == 0)
-    mbox->flags &= ~_MU_MAILBOX_OPEN;
+    {
+      if (mbox->notify_fd >= 0)
+	close (mbox->notify_fd);
+      mbox->flags &= ~_MU_MAILBOX_OPEN;
+    }
   return rc;
 }
 
@@ -314,42 +382,56 @@ mu_mailbox_remove (mu_mailbox_t mbox)
   if (mbox->flags & _MU_MAILBOX_REMOVED)
     return MU_ERR_MBX_REMOVED;
   if (!mbox->_remove)
-    return MU_ERR_EMPTY_VFN;
+    {
+      /* Try the owning folder delete method.  See comment to mu_folder_delete
+	 in folder.c.  This may result in a recursive call to mu_mailbox_remove
+	 which is blocked by setting the _MU_MAILBOX_REMOVED flag. */
+
+      int rc;
+      const char *path;
+      
+      rc = mu_url_sget_path (mbox->url, &path);
+      if (rc == 0)
+	{
+	  mbox->flags |= _MU_MAILBOX_REMOVED;
+	  rc = mu_folder_delete (mbox->folder, path);
+	  if (rc)
+	    mbox->flags &= ~_MU_MAILBOX_REMOVED;
+	}
+      return rc;
+    }
   return mbox->_remove (mbox);
 }
 
 int
 mu_mailbox_flush (mu_mailbox_t mbox, int expunge)
 {
-  size_t i, total = 0;
-  int status = 0;
-  
   if (!mbox)
     return EINVAL;
   if (mbox->flags & _MU_MAILBOX_REMOVED)
     return MU_ERR_MBX_REMOVED;
   if (!(mbox->flags & _MU_MAILBOX_OPEN))
     return _MU_MAILBOX_OPEN;
-  if (!(mbox->flags & (MU_STREAM_RDWR|MU_STREAM_WRITE|MU_STREAM_APPEND)))
+  if (!(mbox->flags & (MU_STREAM_WRITE|MU_STREAM_APPEND)))
     return 0;
 
-  mu_mailbox_messages_count (mbox, &total);
   if (!(mbox->flags & MU_STREAM_APPEND))
-    for (i = 1; i <= total; i++)
-      {
-	mu_message_t msg = NULL;
-	mu_attribute_t attr = NULL;
-	mu_mailbox_get_message (mbox, i, &msg);
-	mu_message_get_attribute (msg, &attr);
-	mu_attribute_set_seen (attr);
-      }
-
-  if (expunge)
-    status = mu_mailbox_expunge (mbox);
-  else
-    status = mu_mailbox_sync (mbox);
-
-  return status;
+    {
+      size_t i, total = 0;
+  
+      mu_mailbox_messages_count (mbox, &total);
+      for (i = 1; i <= total; i++)
+	{
+	  mu_message_t msg = NULL;
+	  mu_attribute_t attr = NULL;
+	  mu_mailbox_get_message (mbox, i, &msg);
+	  mu_message_get_attribute (msg, &attr);
+	  mu_attribute_set_seen (attr);
+	}
+      if (expunge)
+	return mu_mailbox_expunge (mbox);
+    }
+  return mu_mailbox_sync (mbox);
 }
 
 #define _MBOX_CHECK_FLAGS(mbox)			\
@@ -358,7 +440,7 @@ mu_mailbox_flush (mu_mailbox_t mbox, int expunge)
   if (mbox->flags & _MU_MAILBOX_REMOVED)	\
     return MU_ERR_MBX_REMOVED;			\
   if (!(mbox->flags & _MU_MAILBOX_OPEN))	\
-    return _MU_MAILBOX_OPEN
+    return MU_ERR_NOT_OPEN
 
 #define _MBOX_CHECK(mbox,method)		\
   _MBOX_CHECK_FLAGS(mbox);			\
@@ -373,11 +455,18 @@ mu_mailbox_flush (mu_mailbox_t mbox, int expunge)
 /* messages */
 int
 mu_mailbox_append_message (mu_mailbox_t mbox, mu_message_t msg)
+{
+  return mu_mailbox_append_message_ext (mbox, msg, NULL, NULL);
+}
+
+int
+mu_mailbox_append_message_ext (mu_mailbox_t mbox, mu_message_t msg,
+			       mu_envelope_t env, mu_attribute_t atr)
 {
   _MBOX_CHECK_Q (mbox, _append_message);
-  if (!(mbox->flags & (MU_STREAM_RDWR|MU_STREAM_WRITE|MU_STREAM_APPEND)))
+  if (!(mbox->flags & (MU_STREAM_WRITE|MU_STREAM_APPEND)))
     return EACCES;
-  return mbox->_append_message (mbox, msg);
+  return mbox->_append_message (mbox, msg, env, atr);
 }
 
 int
@@ -407,38 +496,90 @@ mu_mailbox_messages_count (mu_mailbox_t mbox, size_t *num)
 int
 mu_mailbox_messages_recent (mu_mailbox_t mbox, size_t *num)
 {
-  _MBOX_CHECK_Q (mbox, _messages_recent);
-  return mbox->_messages_recent (mbox, num);
+  size_t i, count, n;
+  int rc;
+
+  _MBOX_CHECK_FLAGS (mbox);
+  if (mbox->flags & MU_STREAM_QACCESS)
+    return MU_ERR_BADOP;
+  if (mbox->_messages_recent)
+    return mbox->_messages_recent (mbox, num);
+
+  rc = mu_mailbox_messages_count (mbox, &count);
+  if (rc)
+    return rc;
+  n = 0;
+  for (i = 1; i < count; i++)
+    {
+      mu_message_t msg;
+      mu_attribute_t attr;
+
+      rc = mu_mailbox_get_message (mbox, i, &msg);
+      if (rc)
+	return rc;
+      rc = mu_message_get_attribute (msg, &attr);
+      if (rc)
+	return rc;
+      if (mu_attribute_is_recent (attr))
+	n++;
+    }
+  *num = n;
+  return 0;
 }
 
 int
 mu_mailbox_message_unseen (mu_mailbox_t mbox, size_t *num)
 {
-  _MBOX_CHECK_Q (mbox, _message_unseen);
-  return mbox->_message_unseen (mbox, num);
+  int rc;
+  size_t i, count;
+
+  _MBOX_CHECK_FLAGS (mbox);
+  if (mbox->flags & MU_STREAM_QACCESS)
+    return MU_ERR_BADOP;
+  if (mbox->_message_unseen)
+    return mbox->_message_unseen (mbox, num);
+
+  rc = mu_mailbox_messages_count (mbox, &count);
+  if (rc)
+    return rc;
+  for (i = 1; i < count; i++)
+    {
+      mu_message_t msg;
+      mu_attribute_t attr;
+      int rc;
+
+      rc = mu_mailbox_get_message (mbox, i, &msg);
+      if (rc)
+	return rc;
+      rc = mu_message_get_attribute (msg, &attr);
+      if (rc)
+	return rc;
+      if (!mu_attribute_is_seen (attr))
+	{
+	  *num = i;
+	  return 0;
+	}
+    }
+
+  *num = 0;
+  return 0;
 }
 
 int
 mu_mailbox_sync (mu_mailbox_t mbox)
 {
   _MBOX_CHECK_Q (mbox, _sync);
-  if (!(mbox->flags & (MU_STREAM_RDWR|MU_STREAM_WRITE|MU_STREAM_APPEND)))
+  if (!(mbox->flags & (MU_STREAM_WRITE|MU_STREAM_APPEND)))
     return 0;
   return mbox->_sync (mbox);
 }
 
 /* Historic alias: */
-int
-mu_mailbox_save_attributes (mu_mailbox_t mbox)
-{
-  return mu_mailbox_sync (mbox);
-}
-
 int
 mu_mailbox_expunge (mu_mailbox_t mbox)
 {
   _MBOX_CHECK_Q (mbox, _expunge);
-  if (!(mbox->flags & (MU_STREAM_RDWR|MU_STREAM_WRITE|MU_STREAM_APPEND)))
+  if (!(mbox->flags & (MU_STREAM_WRITE|MU_STREAM_APPEND)))
     return EACCES;
   return mbox->_expunge (mbox);
 }
@@ -501,8 +642,15 @@ mu_mailbox_get_size (mu_mailbox_t mbox, mu_off_t *psize)
 int
 mu_mailbox_uidvalidity (mu_mailbox_t mbox, unsigned long *pvalid)
 {
-  _MBOX_CHECK_Q (mbox, _uidvalidity);
-  return mbox->_uidvalidity (mbox, pvalid);
+  _MBOX_CHECK_Q (mbox, _get_uidvalidity);
+  return mbox->_get_uidvalidity (mbox, pvalid);
+}
+
+int
+mu_mailbox_uidvalidity_reset (mu_mailbox_t mbox)
+{
+  _MBOX_CHECK_Q (mbox, _set_uidvalidity);
+  return mbox->_set_uidvalidity (mbox, time (NULL));
 }
 
 int
@@ -559,55 +707,6 @@ mu_mailbox_set_stream (mu_mailbox_t mbox, mu_stream_t stream)
   return 0;
 }
 
-/* FIXME: This is a problem.  We provide a mu_mailbox_get_stream ()
-   and this stream is special: it should, in theory, represent
-   a "view" of a flow of messages.  However, providing this perspective
-   may make sense for local mailboxes, but downright impossible
-   for remote mailboxes, short on downloading entire mailbox
-   locally.
-
-   This function will be removed in the short run.  It is no longer
-   used by MU.  */
-int
-mu_mailbox_get_stream (mu_mailbox_t mbox, mu_stream_t *pstream)
-{
-  /* FIXME: Deprecation warning */
-  if (mbox == NULL)
-    return EINVAL;
-  if (pstream == NULL)
-    return MU_ERR_OUT_PTR_NULL;
-
-  /* If null two cases:
-     - it is no open yet.
-     - it a remote stream and the socket stream is on the folder.  */
-  if (mbox->stream == NULL)
-    {
-      if (mbox->folder)
-	return mu_folder_get_stream (mbox->folder, pstream);
-    }
-  *pstream = mbox->stream;
-  return 0;
-}
-
-int
-mu_mailbox_get_streamref (mu_mailbox_t mbox, mu_stream_t *pstream)
-{
-  if (mbox == NULL)
-    return EINVAL;
-  if (pstream == NULL)
-    return MU_ERR_OUT_PTR_NULL;
-
-  /* If null two cases:
-     - it is no open yet.
-     - it a remote stream and the socket stream is on the folder.  */
-  if (mbox->stream == NULL)
-    {
-      if (mbox->folder)
-	return mu_folder_get_streamref (mbox->folder, pstream);
-    }
-  return mu_streamref_create (pstream, mbox->stream);
-}
-
 int
 mu_mailbox_get_observable (mu_mailbox_t mbox, mu_observable_t *pobservable)
 {
@@ -689,7 +788,7 @@ mu_mailbox_set_folder (mu_mailbox_t mbox, mu_folder_t folder)
 {
   if (mbox == NULL)
     return EINVAL;
-   mbox->folder = folder;
+  mbox->folder = folder;
   return 0;
 }
 
@@ -752,7 +851,7 @@ mu_mailbox_get_uidls (mu_mailbox_t mbox, mu_list_t *plist)
 	      break;
 	    }
 	  uidl->msgno = i;
-	  strncpy (uidl->uidl, strdup (buf), MU_UIDL_BUFFER_SIZE);
+	  strncpy (uidl->uidl, buf, MU_UIDL_BUFFER_SIZE);
 	  status = mu_list_append (list, uidl);
 	  if (status)
 	    {
@@ -840,7 +939,7 @@ _search_message_uid (mu_mailbox_t mbox, size_t uid, size_t *result)
   return _uid_bsearch (mbox, 1, count, uid, result);
 }
 
-/* Translat message UIDs to message numbers and vice versa. */
+/* Translate message UIDs to message numbers and vice versa. */
 int
 mu_mailbox_translate (mu_mailbox_t mbox, int cmd, size_t from, size_t *to)
 {
@@ -878,3 +977,12 @@ mu_mailbox_translate (mu_mailbox_t mbox, int cmd, size_t from, size_t *to)
   return rc;
 }
 
+int
+mu_mailbox_access_time (mu_mailbox_t mbox, time_t *return_time)
+{
+  _MBOX_CHECK_Q (mbox, _get_atime);
+  if (!return_time)
+    return MU_ERR_OUT_PTR_NULL;
+  return mbox->_get_atime (mbox, return_time);
+}
+
