@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2020 Free Software Foundation, Inc.
+   Copyright (C) 1999-2024 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -146,7 +146,7 @@ mu_mailer_create_from_url (mu_mailer_t *pmailer, mu_url_t url)
 }
 
 int
-mu_mailer_create (mu_mailer_t * pmailer, const char *name)
+mu_mailer_create (mu_mailer_t *pmailer, const char *name)
 {
   int status;
   mu_url_t url;
@@ -184,13 +184,6 @@ mu_mailer_destroy (mu_mailer_t * pmailer)
 
       mu_monitor_wrlock (monitor);
 
-      if (mailer->stream)
-	{
-	  /* FIXME: Should be the client responsability to close this?  */
-	  /* mu_stream_close (mailer->stream); */
-	  mu_stream_destroy (&mailer->stream);
-	}
-
       if (mailer->url)
 	mu_url_destroy (&(mailer->url));
 
@@ -629,6 +622,7 @@ send_fragments (mu_mailer_t mailer,
 	break;
 
       mu_mime_to_message (mime, &newmsg);
+      mu_mime_destroy (&mime);
       merge_headers (newmsg, hdr);
       
       status = mailer->_send_message (mailer, newmsg, from, to);
@@ -722,25 +716,6 @@ mu_mailer_send_message (mu_mailer_t mailer, mu_message_t msg,
   return mu_mailer_send_fragments (mailer, msg, 0, NULL, from, to);
 }
 
-int
-mu_mailer_set_stream (mu_mailer_t mailer, mu_stream_t stream)
-{
-  if (mailer == NULL)
-    return EINVAL;
-  mailer->stream = stream;
-  return 0;
-}
-
-int
-mu_mailer_get_streamref (mu_mailer_t mailer, mu_stream_t * pstream)
-{
-  if (mailer == NULL)
-    return EINVAL;
-  if (pstream == NULL)
-    return MU_ERR_OUT_PTR_NULL;
-  return mu_streamref_create (pstream, mailer->stream);
-}
-
 int
 mu_mailer_get_observable (mu_mailer_t mailer, mu_observable_t * pobservable)
 {
