@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2011-2020 Free Software Foundation, Inc.
+   Copyright (C) 2011-2024 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -76,7 +76,6 @@ mu_sockaddr_from_node (struct mu_sockaddr **retval, const char *node,
 				 sizeof (s_un));
     }
   else 
-#ifdef MAILUTILS_IPV6
     {
       struct addrinfo hints;
       struct addrinfo *res, *ap;
@@ -162,95 +161,5 @@ mu_sockaddr_from_node (struct mu_sockaddr **retval, const char *node,
 	  }
       freeaddrinfo (res);
     }
-#else
-  if (mh->family == AF_INET || mh->family == AF_UNSPEC)
-    {
-      short port;
-      struct hostent *hp;
-      struct mu_sockaddr *tail = NULL;
-      char **p;
-
-      if (serv)
-	{
-	  char *end;
-	  unsigned long n = strtoul (serv, &end, 10);
-
-	  if (*end)
-	    {
-	      struct servent *sp;
-	      const char *proto;
-
-	      if (mh->protocol)
-		{
-		  struct protoent *pp = getprotobynumber (mh->protocol);
-		  if (!pp)
-		    return EINVAL;
-		  proto = pp->p_name;
-		}
-	      else
-		proto = NULL;
-	      
-	      sp = getservbyname (serv, proto);
-	      if (!sp)
-		return MU_ERR_SERVICE;
-	      port = sp->s_port;
-	    }
-	  else if (n == 0 || n > USHRT_MAX)
-	    return ERANGE;
-	  else 
-	    port = htons (n);
-	}
-      else if (mh->port)
-	port = htons (mh->port);
-      else
-	port = 0;
-      
-      if (!node)
-	{
-	  struct sockaddr_in s_in;
-
-	  if (!(mh->flags & MU_AH_PASSIVE))
-	    return MU_ERR_NONAME;
-
-	  s_in.sin_family = AF_INET;
-	  s_in.sin_addr.s_addr = INADDR_ANY;
-	  s_in.sin_port = port;
-	  return mu_sockaddr_create (retval, (struct sockaddr*)&s_in,
-				     sizeof (s_in));
-	}
-      
-      hp = gethostbyname (node);
-      if (!hp)
-	return MU_ERR_GETHOSTBYNAME;
-
-      if (hp->h_addrtype != AF_INET || hp->h_length != 4)
-	return MU_ERR_FAMILY;
-
-      for (p = hp->h_addr_list; *p; p++)
-	{
-	  struct mu_sockaddr *sa;
-	  struct sockaddr_in s_in;
-
-	  s_in.sin_family = AF_INET;
-	  memcpy(&s_in.sin_addr, *p, 4);
-	  s_in.sin_port = port;
-	    
-	  rc = mu_sockaddr_create (&sa, (struct sockaddr*)&s_in,
-				   sizeof (s_in));
-	  if (rc)
-	    {
-	      mu_sockaddr_free_list (*retval);
-	      return rc;
-	    }
-	  if (tail)
-	    mu_sockaddr_insert (tail, sa, 0);
-	  else
-	    *retval = sa;
-	  tail = sa;
-	}
-    }
-  else
-    return MU_ERR_FAMILY;
-#endif
   return 0;
 }
