@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2012, 2014-2017 Free Software Foundation, Inc.
+   Copyright (C) 2010-2024 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -24,18 +24,6 @@
 #include <mailutils/errno.h>
 #include <mailutils/sys/streamref.h>
 
-#define _MU_STR_ERRMASK (_MU_STR_ERR|_MU_STR_EOF)
-
-static int
-streamref_return (struct _mu_streamref *sp, int rc)
-{
-  if (rc)
-    sp->stream.last_err = sp->transport->last_err;
-  sp->stream.flags = (sp->stream.flags & ~_MU_STR_ERRMASK) |
-                   (sp->transport->flags & _MU_STR_ERRMASK);
-  return rc;
-}
-
 static int
 _streamref_read (struct _mu_stream *str, char *buf, size_t bufsize,
 		 size_t *pnread)
@@ -67,41 +55,7 @@ _streamref_read (struct _mu_stream *str, char *buf, size_t bufsize,
       mu_stream_clearerr (sp->transport);
       return 0;
     }
-  return streamref_return (sp, rc);
-}
-
-static int
-_streamref_readdelim (struct _mu_stream *str, char *buf, size_t bufsize,
-		      int delim, size_t *pnread)
-{
-  struct _mu_streamref *sp = (struct _mu_streamref *)str;
-  int rc;
-  size_t nread;
-  mu_off_t off;
-  
-  rc = mu_stream_seek (sp->transport, sp->offset, MU_SEEK_SET, &off);
-  if (rc == 0)
-    {
-      rc = mu_stream_readdelim (sp->transport, buf, bufsize, delim, &nread);
-      if (rc == 0)
-	{
-	  if (sp->end)
-	    {
-	      size_t size = sp->end - off + 1;
-	      if (nread > size)
-		nread = size;
-	    }
-	  sp->offset += nread;
-	  *pnread = nread;
-	}
-    }
-  else if (rc == ESPIPE)
-    {
-      *pnread = 0;
-      mu_stream_clearerr (sp->transport);
-      return 0;
-    }
-  return streamref_return (sp, rc);
+  return rc;
 }
 
 static int
@@ -121,28 +75,28 @@ _streamref_write (struct _mu_stream *str, const char *buf, size_t bufsize,
 	  *pnwrite = nwrite;
 	}
     }
-  return streamref_return (sp, rc);
+  return rc;
 }
 
 static int
 _streamref_flush (struct _mu_stream *str)
 {
   struct _mu_streamref *sp = (struct _mu_streamref *)str;
-  return streamref_return (sp, mu_stream_flush (sp->transport));
+  return mu_stream_flush (sp->transport);
 }
 
 static int
 _streamref_open (struct _mu_stream *str)
 {
   struct _mu_streamref *sp = (struct _mu_streamref *)str;
-  return streamref_return (sp, mu_stream_open (sp->transport));
+  return mu_stream_open (sp->transport);
 }
 
 static int
 _streamref_close (struct _mu_stream *str)
 {
   struct _mu_streamref *sp = (struct _mu_streamref *)str;
-  return streamref_return (sp, mu_stream_close (sp->transport));
+  return mu_stream_close (sp->transport);
 }
 
 static void
@@ -165,7 +119,7 @@ _streamref_seek (struct _mu_stream *str, mu_off_t off, mu_off_t *ppos)
     {
       rc = mu_stream_size (sp->transport, &size);
       if (rc)
-	return streamref_return (sp, rc);
+	return rc;
       size -= sp->start;
     }
   
@@ -174,7 +128,7 @@ _streamref_seek (struct _mu_stream *str, mu_off_t off, mu_off_t *ppos)
   rc = mu_stream_seek (sp->transport, sp->start + off, MU_SEEK_SET,
 		       &sp->offset);
   if (rc)
-    return streamref_return (sp, rc);
+    return rc;
   *ppos = sp->offset - sp->start;
   return 0;
 }
@@ -192,7 +146,7 @@ _streamref_size (struct _mu_stream *str, mu_off_t *psize)
     {
       rc = mu_stream_size (sp->transport, &size);
       if (rc)
-	return streamref_return (sp, rc);
+	return rc;
       size -= sp->start;
     }
   if (rc == 0)
@@ -231,29 +185,28 @@ _streamref_ctl (struct _mu_stream *str, int code, int opcode, void *arg)
 	    }
 	}
     }
-  return streamref_return (sp, mu_stream_ioctl (sp->transport, code,
-						opcode, arg));
+  return mu_stream_ioctl (sp->transport, code, opcode, arg);
 }
 
 static int
 _streamref_wait (struct _mu_stream *str, int *pflags, struct timeval *tvp)
 {
   struct _mu_streamref *sp = (struct _mu_streamref *)str;
-  return streamref_return (sp, mu_stream_wait (sp->transport, pflags, tvp));
+  return mu_stream_wait (sp->transport, pflags, tvp);
 }
 
 static int
 _streamref_truncate (struct _mu_stream *str, mu_off_t size)
 {
   struct _mu_streamref *sp = (struct _mu_streamref *)str;
-  return streamref_return (sp, mu_stream_truncate (sp->transport, size));
+  return mu_stream_truncate (sp->transport, size);
 }
   
 static int
 _streamref_shutdown (struct _mu_stream *str, int how)
 {
   struct _mu_streamref *sp = (struct _mu_streamref *)str;
-  return streamref_return (sp, mu_stream_shutdown (sp->transport, how));
+  return mu_stream_shutdown (sp->transport, how);
 }
 
 static const char *
@@ -270,14 +223,9 @@ int
 mu_streamref_create_abridged (mu_stream_t *pref, mu_stream_t str,
 			      mu_off_t start, mu_off_t end)
 {
-  int rc;
-  mu_off_t off;
   int flags;
   struct _mu_streamref *sp;
   
-  rc = mu_stream_seek (str, 0, MU_SEEK_SET, &off);/*FIXME: SEEK_CUR?*/
-  if (rc)
-    return rc;
   mu_stream_get_flags (str, &flags);
   sp = (struct _mu_streamref *)
          _mu_stream_create (sizeof (*sp), flags | _MU_STR_OPEN);
@@ -287,8 +235,6 @@ mu_streamref_create_abridged (mu_stream_t *pref, mu_stream_t str,
   mu_stream_ref (str);
 
   sp->stream.read = _streamref_read; 
-  if (str->readdelim)
-    sp->stream.readdelim = _streamref_readdelim; 
   sp->stream.write = _streamref_write;
   sp->stream.flush = _streamref_flush;
   sp->stream.open = _streamref_open; 
@@ -305,9 +251,7 @@ mu_streamref_create_abridged (mu_stream_t *pref, mu_stream_t str,
   sp->transport = str;
   sp->start = start;
   sp->end = end;
-  if (off < start || off > end)
-    off = start;
-  sp->offset = off;
+  sp->offset = start;
   *pref = (mu_stream_t) sp;
 
   mu_stream_set_buffer (*pref, mu_buffer_full, 0);
