@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2011-2020 Free Software Foundation, Inc.
+   Copyright (C) 2011-2024 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -261,6 +261,18 @@ _gdbm_nextkey (mu_dbm_file_t db, struct mu_dbm_datum *ret)
   return 0;
 }
 
+static int
+_gdbm_break (mu_dbm_file_t db)
+{
+  struct gdbm_descr *gd = db->db_descr;
+  if (!gd->prev.dptr)
+    return MU_ERR_NOENT;
+  free (gd->prev.dptr);
+  gd->prev.dptr = NULL;
+  gd->prev.dsize = 0;
+  return 0;
+}
+
 static void
 _gdbm_datum_free (struct mu_dbm_datum *datum)
 {
@@ -285,6 +297,7 @@ struct mu_dbm_impl _mu_dbm_gdbm = {
   _gdbm_firstkey,
   _gdbm_nextkey,
   _gdbm_datum_free,
-  _gdbm_strerror
+  _gdbm_strerror,
+  _gdbm_break
 };
 #endif
