@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2011-2020 Free Software Foundation, Inc.
+   Copyright (C) 2011-2024 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -870,9 +870,7 @@ _fill_response (void *item, void *data)
   switch (env->n++)
     {
     case env_date:
-      if (elt->type != imap_eltype_string)
-	rc = MU_ERR_FAILURE;
-      else
+      if (elt->type == imap_eltype_string)
 	{
 	  if (mu_scan_datetime (elt->v.string,
 				MU_DATETIME_SCAN_RFC822,
@@ -882,6 +880,10 @@ _fill_response (void *item, void *data)
 	  else
 	    rc = 0;
 	}
+      else if (_mu_imap_list_element_is_nil (elt))
+	rc = 0;
+      else
+	rc = MU_ERR_FAILURE;
       break;
       
     case env_subject:
@@ -919,6 +921,9 @@ _fill_response (void *item, void *data)
     case env_message_id:
       rc = elt_to_string (elt, &imapenvelope->message_id);
       break;
+
+    default:
+      rc = EINVAL;
     }
   return rc;
 }
