@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2005, 2007, 2010-2012, 2014-2018 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2024 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,7 +24,8 @@
 int
 mail_cd (int argc, char **argv)
 {
-  char *dir;
+  char *dir, *edir;
+  int rc;
   
   if (argc > 2)
     return 1;
@@ -34,10 +34,16 @@ mail_cd (int argc, char **argv)
   else 
     dir = getenv ("HOME");
 
-  if (chdir (dir))
+  rc = mu_mailbox_expand_name (dir, &edir);
+  if (rc)
     {
-      mu_diag_funcall (MU_DIAG_ERROR, "chdir", dir, errno);
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_mailbox_expand_name", dir, rc);
       return 1;
     }
-  return 0;
+
+  rc = chdir (edir);
+  if (rc)
+    mu_diag_funcall (MU_DIAG_ERROR, "chdir", edir, errno);
+  free (edir);
+  return rc;
 }
