@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2006-2007, 2010-2012, 2014-2017 Free Software
-   Foundation, Inc.
+   Copyright (C) 2006-2024 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -85,9 +84,17 @@ enum mh_opcode
   /* Set format specification.
      Format: mhop_fmtspec number */
   mhop_fmtspec,
+
+  /* Push numeric register */
+  mhop_pushn,
+  /* Pop numeric register */
+  mhop_popn,
+  /* Exchange top of stack value and numeric register */
+  mhop_xchgn,
 };    
 
 enum regid { R_REG, R_ARG, R_ACC };
+#define MH_NREG 3
 
 enum mh_type
 {
@@ -158,8 +165,12 @@ struct mh_string
   
 struct mh_fvm
 {
-  long num[2];              /* numeric registers */
-  struct mh_string str[3];  /* string registers */
+  long num[MH_NREG];              /* numeric registers */
+  struct mh_string str[MH_NREG];  /* string registers */
+
+  long *numstack;           /* Stack of numeric value */
+  size_t maxstack;          /* Stack capacity */
+  size_t tos;               /* Top of stack (next free slot) */
   
   size_t pc;                /* Program counter */
   size_t progcnt;           /* Size of allocated program*/
