@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2022 Free Software Foundation, Inc.
+   Copyright (C) 2010-2024 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -544,7 +544,7 @@ com_send (int argc, char **argv)
     {
       if (msgfile)
 	{
-	  switch (mu_getans ("rReEdD",
+	  switch (mu_getans ("rReEuUdD",
 			     _("Previous message exists. "
 			       "What now: [r]euse, [e]dit, "
 			       "[u]se as a template or\n"
