@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2007, 2009-2012, 2014-2017 Free Software
-   Foundation, Inc.
+   Copyright (C) 2003-2024 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,18 +18,19 @@
 
 /* STLS command -- TLS/SSL encryption */
 
-#ifdef WITH_TLS
-
 int
 pop3d_stls (char *arg, struct pop3d_session *session)
 {
+  if (session->tls_mode == tls_no)
+    return ERR_BAD_CMD;
+
   if (strlen (arg) != 0)
     return ERR_BAD_ARGS;
 
   if (state != initial_state)
     return ERR_WRONG_STATE;
 
-  switch (session->tls)
+  switch (session->tls_mode)
     {
     case tls_ondemand:
     case tls_required:
@@ -38,27 +38,20 @@ pop3d_stls (char *arg, struct pop3d_session *session)
     default:
       return ERR_WRONG_STATE;
     }
-  
-  if (tls_done)
-    return ERR_TLS_ACTIVE;
 
   pop3d_outf ("+OK Begin TLS negotiation\n");
   pop3d_flush_output ();
 
-  tls_done = pop3d_init_tls_server () == 0;
-
-  if (!tls_done)
+  if (pop3d_init_tls_server (session->tls_conf))
     {
       mu_diag_output (MU_DIAG_ERROR, _("Session terminated"));
       state = ABORT;
       return ERR_UNKNOWN;
     }
 
+  session->tls_mode = tls_no;
   state = AUTHORIZATION;  /* Confirm we're in this state. Necessary for
 			     "tls required" to work */
 
   return OK;
 }
-
-#endif /* WITH_TLS */
-
