@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009-2012 Free Software Foundation, Inc.
+   Copyright (C) 2009-2024 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -19,6 +19,8 @@
 # include <config.h>
 #endif
 
+#include <Python.h>
+
 #include <mailutils/address.h>
 #include <mailutils/attribute.h>
 #include <mailutils/auth.h>
@@ -52,12 +54,23 @@
 #define PY_ROOT_NAME "c_api"
 #define PY_PACKAGE_VERSION PACKAGE_VERSION
 
-extern inline PyObject * _ro (PyObject *obj);
+#define ASSERT_INDEX_RANGE(n, t)					 \
+  do									 \
+    {									 \
+      if ((n) <= 0)							 \
+	{								 \
+          PyErr_SetString (PyExc_RuntimeError, t " index out of range"); \
+          return NULL;					   	         \
+	}								 \
+    }									 \
+  while (0)
+
+extern PyObject * _ro (PyObject *obj);
 extern void _py_dealloc (PyObject *self);
 extern PyObject * status_object (int status, PyObject *py_obj);
-extern PyObject * _mu_py_attach_module (const char *name, PyMethodDef *methods);
+extern PyObject * _mu_py_attach_module (struct PyModuleDef *mdef);
 
-extern void mu_py_attach_modules (void);
+extern PyObject *mu_py_attach_modules (void);
 
 extern void _mu_py_attach_error (void);
 extern void _mu_py_attach_address (void);
